esqueleto-postgis-4.0.0: postgis bindings for esqueleto.
Safe HaskellNone
LanguageHaskell2010

Database.Esqueleto.Postgis.Wkb

Description

Ewkb is deeply integrated into Wkb, which is why we kept this around. For postgis we mostly want Ewkb.

This module Allows parsing of ByteString into a Geospatial Object, and provides foundational elements for constructing an Ewkb.

Refer to the WKB Wikipedia page https://en.wikipedia.org/wiki/Well-known_text#Well-known_binary

Synopsis

Documentation

parseByteString :: ByteString -> Either String GeospatialGeometry Source #

Representation of WKB as Binary

parseHexByteString :: Base16 ByteString -> Either String GeospatialGeometry Source #

Representation of WKB as a String in Base16/Hex form i.e. "0101000000000000000000f03f0000000000000040" is POINT 1.0 2.0

toByteString :: EndianType -> GeospatialGeometry -> ByteString Source #

Produce the binary representation of WKB given its EndianType (Little or Big - Intel is Little). Use EWKB when you know the SRID.

Core

the greasy gears inside for experienced users or the brave!

Geometry

Endian

data EndianType Source #

Constructors

LittleEndian 
BigEndian 

Instances

Instances details
Show EndianType Source # 
Instance details

Defined in Database.Esqueleto.Postgis.Wkb.Endian

Eq EndianType Source # 
Instance details

Defined in Database.Esqueleto.Postgis.Wkb.Endian

Point